/* $Id: vms.c,v 1.11 1999/06/22 12:32:07 levitte Exp $ */

/* This file contains all the VMS-specific things we might need */

#include "gnu_extras.h"

#include <string.h>
#include <descrip.h>
#include <ttdef.h>
#include <iodef.h>
#include <stsdef.h>
#include <syidef.h>
#include <lnmdef.h>
#include <ssdef.h>
#include <lib$routines.h>
#include <starlet.h>
#include "fish.h"
#include "vms.h"
#include "util.h"

struct iosb_t {
    unsigned short status;
    unsigned short buflen;
    unsigned short terminator;
    unsigned short terminatorsize;
};

#define INIT_DSC2(dsc,str,len) do { (dsc).dsc$w_length = (len); \
                                    (dsc).dsc$b_dtype = DSC$K_DTYPE_T; \
                                    (dsc).dsc$b_class = DSC$K_CLASS_S; \
                                    (dsc).dsc$a_pointer = (str); } while(0)
#define INIT_DSC(dsc,str)      INIT_DSC2((dsc), (str), (strlen(str)))

struct itemlist3_t {
    unsigned short buflen;
    unsigned short itemcode;
    void *bufaddr;
    unsigned long *resultlen;
};



/* TTY specific things */

tty_s *tty_open(const char *name, int *status)
{
    tty_s *tty = xmalloc(sizeof(tty_s));
    INIT_DSC2(tty->name, xstrdup(name), strlen(name));
    
    *status = sys$assign(&(tty->name),&(tty->channel),0,0);
    if (!(*status & 1)) {
	xfree(tty);
    }
    return tty;
}
int tty_close(tty_s *tty, int *status)
{
    sys$dassgn(tty->channel);
    tty_release(tty,status);
    return 1;
}
int tty_release(tty_s *tty, int *status)
{
    xfree(tty->name.dsc$a_pointer);
    xfree(tty);
    return 1;
}

int tty_io_cancel(tty_s *tty, int *status)
{
    return (*status = sys$cancel(tty->channel)) & 1;
}

tty_s *tty_dup(tty_s *tty)
{
    tty_s *tty2 = xmalloc(sizeof(tty_s));
    memcpy(tty2, tty, sizeof(tty_s));
    tty2->name.dsc$a_pointer = xstrdup(tty->name.dsc$a_pointer);
    return tty2;
}

int tty_setflags(tty_s *tty, int flags, int flags2, int *status)
{
    tty->tty_char.basic_chars |= flags;
    tty->tty_char.extended_chars |= flags2;
    return 1;
}

int tty_clearflags(tty_s *tty, int flags, int flags2, int *status)
{
    tty->tty_char.basic_chars &= ~(flags);
    tty->tty_char.extended_chars &= ~(flags2);
    return 1;
}

int tty_copychar(tty_s *tty, tty_s *from)
{
    memcpy(&tty->tty_char,&from->tty_char,sizeof(from->tty_char));
    return 1;
}

int tty_sensemode(tty_s *tty, int *status)
{
    return (*status = sys$qiow(0, tty->channel, IO$_SENSEMODE, 0, 0, 0,
			       &tty->tty_char, sizeof(tty->tty_char),
			       0, 0, 0, 0)) & 1;
}

int tty_setmode(tty_s *tty, int *status)
{
    return (*status = sys$qiow(0, tty->channel, IO$_SETMODE, 0, 0, 0,
			       &tty->tty_char, sizeof(tty->tty_char),
			       0, 0, 0, 0)) & 1;
}

tty_s *tty_noecho(const char *name, int *status)
{
    tty_s *tty = tty_open(name, status);
    tty_s *tty2 = 0;
    int dummy;
    if (!tty_sensemode(tty, status)) {
	tty_release(tty, &dummy);
	tty_close(tty2, &dummy);
	return 0;
    }
    tty2 = tty_dup(tty);
    tty_setflags(tty,TT$M_NOECHO,0,&dummy);
    if (!tty_setmode(tty, status)) {
	tty_release(tty, &dummy);
	tty_close(tty2, &dummy);
	return 0;
    }
    tty_release(tty, &dummy);
    return tty2;
}

tty_s *tty_echo(const char *name, int *status)
{
    tty_s *tty = tty_open(name, status);
    tty_s *tty2 = 0;
    int dummy;
    if (!tty_sensemode(tty, status)) {
	tty_release(tty, &dummy);
	tty_close(tty2, &dummy);
	return 0;
    }
    tty2 = tty_dup(tty);
    tty_clearflags(tty,TT$M_NOECHO,0,&dummy);
    if (!tty_setmode(tty, status)) {
	tty_release(tty, &dummy);
	tty_close(tty2, &dummy);
	return 0;
    }
    tty_release(tty, &dummy);
    return tty2;
}

int read_prompted(const char *name, const char *prompt,
		  char *bufaddr, unsigned long bufsize,
		  int purged, int echo, int *status)
{
    tty_s *tty = tty_open(name, status);
    int dummy;
    int function = (purged ? IO$M_PURGE : 0) | (echo ? 0 : IO$M_NOECHO)
	| IO$_READPROMPT;
    unsigned long buflen;
    struct iosb_t iosb;
    struct itemlist3_t il3[] = {
	{ 0, SYI$_MAXBUF, 0, 0 },
	{ 0, 0, 0, 0 },
	};

    il3[0].buflen = sizeof(dummy);
    il3[0].bufaddr = &dummy;
    il3[0].resultlen = 0;
    *status = sys$getsyiw(0, 0, 0, il3, 0, 0, 0);
    if (bufsize > dummy)
	bufsize = dummy;

    /* The logic above should give me what I want, but doesn't, so let's
       settle for something low. */
    if (bufsize > 500)
	bufsize = 500;

    *status = sys$qiow(0, tty->channel, function, &iosb, 0, 0,
		       /* P1 */ bufaddr,
		       /* P2 */ bufsize,
		       /* P3 */ 0,
		       /* P4 */ 0,
		       /* P5 */ prompt,
		       /* P6 */ strlen(prompt));
    tty_close(tty, &dummy);
    tty_release(tty, &dummy);
    if (!$VMS_STATUS_SUCCESS(*status))
	lib$signal(*status);

    *status = iosb.status;
    bufaddr[iosb.buflen] = '\0';
    return $VMS_STATUS_SUCCESS(*status);
}

/* C correspondents to certain VMS routines */

/* like lib$put_output(), but takes a NUL-terminated char* instead of a string
   descriptor */
int lib_put_output(char *str)
{
    struct dsc$descriptor_s dsc;

    INIT_DSC(dsc, str);
    
    return lib$put_output(&dsc);
}

/* like sys$trnlnm(), but only takes the simplest cases */
int sys_trnlnm(char *lognam, char *table, int acmode,
              char *value, int valuesize)
{
    struct itemlist3_t il3[] = {
    { 0, LNM$_STRING, 0, 0 },
    { 0, 0, 0, 0 },
    };
    struct dsc$descriptor_s lognam_dsc;
    struct dsc$descriptor_s table_dsc;
    int status;
    unsigned long resultlen = 0;

    if (table == 0)
       table = "LNM$FILE_DEV";

    if (lognam == 0)
       return SS$_BADPARAM;

    INIT_DSC(table_dsc, table);
    INIT_DSC(lognam_dsc, lognam);
    il3[0].buflen = valuesize;
    il3[0].bufaddr = value;
    il3[0].resultlen = &resultlen;

    status = sys$trnlnm(0, &table_dsc, &lognam_dsc, &acmode, il3);
    value[resultlen] = '\0';

    return status;
}

/* Emacs local variables

Local variables:
eval: (set-c-style "BSD")
end:

*/
